@@ -52,20 +52,24 @@
 #define USEC_PER_SEC 1000000L
 #endif
 
-struct proc_entry {
-	struct cds_list_head entry;
-	struct cds_list_head flows;
-	struct rcu_head rcu;
-
-	struct timeval last_update;
-	unsigned int pid;
-	char name[256];
+struct flow_stat {
 	uint64_t pkts_src, bytes_src;
 	uint64_t pkts_dst, bytes_dst;
 	double rate_bytes_src;
 	double rate_bytes_dst;
 	double rate_pkts_src;
 	double rate_pkts_dst;
+};
+
+struct proc_entry {
+	struct cds_list_head entry;
+	struct cds_list_head flows;
+	struct rcu_head rcu;
+
+	struct timeval last_update;
+	struct flow_stat stat;
+	unsigned int pid;
+	char name[256];
 	int flows_count;
 };
 
@@ -80,8 +84,6 @@ struct flow_entry {
 	uint32_t ip6_src_addr[4], ip6_dst_addr[4];
 	uint16_t port_src, port_dst;
 	uint8_t  tcp_state, tcp_flags, sctp_state, dccp_state;
-	uint64_t pkts_src, bytes_src;
-	uint64_t pkts_dst, bytes_dst;
 	uint64_t timestamp_start, timestamp_stop;
 	char country_src[128], country_dst[128];
 	char country_code_src[4], country_code_dst[4];
@@ -92,10 +94,7 @@ struct flow_entry {
 	bool is_visible;
 	struct nf_conntrack *ct;
 	struct timeval last_update;
-	double rate_bytes_src;
-	double rate_bytes_dst;
-	double rate_pkts_src;
-	double rate_pkts_dst;
+	struct flow_stat stat;
 };
 
 struct flow_list {
@@ -118,8 +117,6 @@ enum flow_direction {
 # define ATTR_TIMESTAMP_STOP 64
 #endif
 
-#define SCROLL_MAX 1000
-
 #define INCLUDE_IPV4	(1 << 0)
 #define INCLUDE_IPV6	(1 << 1)
 #define INCLUDE_UDP	(1 << 2)
@@ -156,7 +153,6 @@ static struct sysctl_params_ctx sysctl = { -1, -1 };
 
 static unsigned int cols, rows;
 static WINDOW *screen;
-static int skip_lines;
 
 static unsigned int interval = 1;
 static bool show_src = false;
@@ -190,12 +186,25 @@ enum tbl_proc_col {
 
 static struct ui_table flows_tbl;
 static struct ui_table procs_tbl;
+static struct ui_table *curr_tbl;
 
 enum tab_entry {
 	TAB_FLOWS,
 	TAB_PROCS,
 };
 
+#define list_first_or_next(__ptr, __head, __entry) \
+({ \
+	struct cds_list_head *h; \
+	if (!__ptr) \
+		h = rcu_dereference((__head)->next); \
+	else if (rcu_dereference(__ptr->__entry.next) == (__head)) \
+		return NULL; \
+	else \
+		h = rcu_dereference(__ptr->__entry.next); \
+	cds_list_entry(h, __typeof(* (__ptr)), __entry); \
+})
+
 static const char *short_options = "vhTUsDIS46ut:nGb";
 static const struct option long_options[] = {
 	{"ipv4",	no_argument,		NULL, '4'},
@@ -369,8 +378,9 @@ static void flow_entry_update_time(struct flow_entry *n)
 	bug_on(gettimeofday(&n->last_update, NULL));
 }
 
-#define CALC_RATE(fld) do {							\
-	n->rate_##fld = (((fld) > n->fld) ? (((fld) - n->fld) / sec) : 0);	\
+#define CALC_RATE(fld) do {					\
+	n->stat.rate_##fld = (((fld) > n->stat.fld) ?		\
+			(((fld) - n->stat.fld) / sec) : 0);	\
 } while (0)
 
 static void flow_entry_calc_rate(struct flow_entry *n, const struct nf_conntrack *ct)
@@ -460,20 +470,24 @@ static struct flow_entry *flow_list_find_id(struct flow_list *fl, uint32_t id)
 	return NULL;
 }
 
+static void __flow_list_del_entry(struct flow_list *fl, struct flow_entry *n)
+{
+	if (n->proc) {
+		cds_list_del_rcu(&n->proc_head);
+		n->proc->flows_count--;
+	}
+
+	cds_list_del_rcu(&n->entry);
+	call_rcu(&n->rcu, flow_entry_xfree_rcu);
+}
+
 static int flow_list_del_entry(struct flow_list *fl, const struct nf_conntrack *ct)
 {
 	struct flow_entry *n;
 
 	n = flow_list_find_id(fl, nfct_get_attr_u32(ct, ATTR_ID));
-	if (n) {
-		if (n->proc) {
-			cds_list_del(&n->proc_head);
-			n->proc->flows_count--;
-		}
-
-		cds_list_del_rcu(&n->entry);
-		call_rcu(&n->rcu, flow_entry_xfree_rcu);
-	}
+	if (n)
+		__flow_list_del_entry(fl, n);
 
 	return NFCT_CB_CONTINUE;
 }
@@ -482,10 +496,8 @@ static void flow_list_destroy(struct flow_list *fl)
 {
 	struct flow_entry *n, *tmp;
 
-	cds_list_for_each_entry_safe(n, tmp, &fl->head, entry) {
-		cds_list_del_rcu(&n->entry);
-		call_rcu(&n->rcu, flow_entry_xfree_rcu);
-	}
+	cds_list_for_each_entry_safe(n, tmp, &fl->head, entry)
+		__flow_list_del_entry(fl, n);
 }
 
 static void proc_list_init(struct proc_list *proc_list)
@@ -546,13 +558,13 @@ static void flow_entry_find_process(struct flow_entry *n)
 	if (snprintf(p->name, sizeof(p->name), "%s", basename(cmdline)) < 0)
 		p->name[0] = '\0';
 
-	p->pkts_src += n->pkts_src;
-	p->pkts_dst += n->pkts_dst;
-	p->bytes_src += n->bytes_src;
-	p->bytes_dst += n->bytes_dst;
+	p->stat.pkts_src += n->stat.pkts_src;
+	p->stat.pkts_dst += n->stat.pkts_dst;
+	p->stat.bytes_src += n->stat.bytes_src;
+	p->stat.bytes_dst += n->stat.bytes_dst;
 	p->flows_count++;
 
-	cds_list_add(&n->proc_head, &p->flows);
+	cds_list_add_rcu(&n->proc_head, &p->flows);
 	n->proc = p;
 }
 
@@ -609,10 +621,10 @@ static void flow_entry_from_ct(struct flow_entry *n, const struct nf_conntrack *
 
 	/* Update stats diff to the related process entry */
 	if (n->proc) {
-		n->proc->pkts_src += pkts_src - n->pkts_src;
-		n->proc->pkts_dst += pkts_dst - n->pkts_dst;
-		n->proc->bytes_src += bytes_src - n->bytes_src;
-		n->proc->bytes_dst += bytes_dst - n->bytes_dst;
+		n->proc->stat.pkts_src += pkts_src - n->stat.pkts_src;
+		n->proc->stat.pkts_dst += pkts_dst - n->stat.pkts_dst;
+		n->proc->stat.bytes_src += bytes_src - n->stat.bytes_src;
+		n->proc->stat.bytes_dst += bytes_dst - n->stat.bytes_dst;
 	}
 
 	CP_NFCT(l3_proto, ATTR_ORIG_L3PROTO, 8);
@@ -631,11 +643,11 @@ static void flow_entry_from_ct(struct flow_entry *n, const struct nf_conntrack *
 	CP_NFCT(sctp_state, ATTR_SCTP_STATE, 8);
 	CP_NFCT(dccp_state, ATTR_DCCP_STATE, 8);
 
-	CP_NFCT(pkts_src, ATTR_ORIG_COUNTER_PACKETS, 64);
-	CP_NFCT(bytes_src, ATTR_ORIG_COUNTER_BYTES, 64);
+	CP_NFCT(stat.pkts_src, ATTR_ORIG_COUNTER_PACKETS, 64);
+	CP_NFCT(stat.bytes_src, ATTR_ORIG_COUNTER_BYTES, 64);
 
-	CP_NFCT(pkts_dst, ATTR_REPL_COUNTER_PACKETS, 64);
-	CP_NFCT(bytes_dst, ATTR_REPL_COUNTER_BYTES, 64);
+	CP_NFCT(stat.pkts_dst, ATTR_REPL_COUNTER_PACKETS, 64);
+	CP_NFCT(stat.bytes_dst, ATTR_REPL_COUNTER_BYTES, 64);
 
 	CP_NFCT(timestamp_start, ATTR_TIMESTAMP_START, 64);
 	CP_NFCT(timestamp_stop, ATTR_TIMESTAMP_STOP, 64);
@@ -1004,54 +1016,54 @@ static void print_flow_peer_info(const struct flow_entry *n, enum flow_direction
 
 	/* Bytes */
 	ui_table_row_col_set(&flows_tbl, TBL_FLOW_BYTES,
-			      bandw2str(SELFLD(dir, bytes_src, bytes_dst),
+			      bandw2str(SELFLD(dir, stat.bytes_src, stat.bytes_dst),
 					tmp, sizeof(tmp) - 1));
 
 	/* Rate bytes */
 	ui_table_row_col_set(&flows_tbl, TBL_FLOW_RATE,
-			      rate2str(SELFLD(dir, rate_bytes_src, rate_bytes_dst),
+			      rate2str(SELFLD(dir, stat.rate_bytes_src, stat.rate_bytes_dst),
 				       tmp, sizeof(tmp) - 1));
 }
 
-static void draw_flow_entry(const struct flow_entry *n)
+static void draw_flow_entry(struct ui_table *tbl, const void *data)
 {
+	const struct flow_entry *n = data;
 	char tmp[128];
 
-	ui_table_row_add(&flows_tbl);
+	ui_table_row_add(tbl);
 
 	/* Application */
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROCESS,
-			      n->proc ?  n->proc->name : "");
+	ui_table_row_col_set(tbl, TBL_FLOW_PROCESS, n->proc ? n->proc->name : "");
 
 	/* PID */
 	slprintf(tmp, sizeof(tmp), "%.d", n->proc ? n->proc->pid : 0);
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PID, tmp);
+	ui_table_row_col_set(tbl, TBL_FLOW_PID, tmp);
 
 	/* L4 protocol */
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROTO, l4proto2str[n->l4_proto]);
+	ui_table_row_col_set(tbl, TBL_FLOW_PROTO, l4proto2str[n->l4_proto]);
 
 	/* L4 protocol state */
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_STATE, flow_state2str(n));
+	ui_table_row_col_set(tbl, TBL_FLOW_STATE, flow_state2str(n));
 
 	/* Time */
 	time2str(n->timestamp_start, tmp, sizeof(tmp));
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_TIME, tmp);
+	ui_table_row_col_set(tbl, TBL_FLOW_TIME, tmp);
 
 	print_flow_peer_info(n, show_src ? FLOW_DIR_SRC : FLOW_DIR_DST);
 
-	ui_table_row_show(&flows_tbl);
+	ui_table_row_show(tbl);
 
 	if (show_src) {
-		ui_table_row_add(&flows_tbl);
+		ui_table_row_add(tbl);
 
-		ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROCESS, "");
-		ui_table_row_col_set(&flows_tbl, TBL_FLOW_PID, "");
-		ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROTO, "");
-		ui_table_row_col_set(&flows_tbl, TBL_FLOW_STATE, "");
-		ui_table_row_col_set(&flows_tbl, TBL_FLOW_TIME, "");
+		ui_table_row_col_set(tbl, TBL_FLOW_PROCESS, "");
+		ui_table_row_col_set(tbl, TBL_FLOW_PID, "");
+		ui_table_row_col_set(tbl, TBL_FLOW_PROTO, "");
+		ui_table_row_col_set(tbl, TBL_FLOW_STATE, "");
+		ui_table_row_col_set(tbl, TBL_FLOW_TIME, "");
 
 		print_flow_peer_info(n, FLOW_DIR_DST);
-		ui_table_row_show(&flows_tbl);
+		ui_table_row_show(tbl);
 	}
 }
 
@@ -1115,10 +1127,10 @@ static inline bool presenter_flow_wrong_state(struct flow_entry *n)
 	return true;
 }
 
-static void draw_filter_status(char *title, int count, int skip_lines)
+static void draw_filter_status(struct ui_table *tbl, char *title)
 {
 	mvwprintw(screen, 1, 0, "%*s", COLS - 1, " ");
-	mvwprintw(screen, 1, 2, "%s(%u) for ", title, count);
+	mvwprintw(screen, 1, 2, "%s(%u) for ", title, ui_table_data_count(tbl));
 
 	if (what & INCLUDE_IPV4)
 		printw("IPv4,");
@@ -1139,60 +1151,31 @@ static void draw_filter_status(char *title, int count, int skip_lines)
 	if (show_active_only)
 		printw("Active,");
 
-	printw(" [+%d]", skip_lines);
+	printw(" [+%d]", ui_table_scroll_height(tbl));
 
 	if (is_flow_collecting)
 		printw(" [Collecting flows ...]");
 
 }
 
-static void draw_flows(WINDOW *screen, struct flow_list *fl,
-		       int skip_lines)
+static void draw_flows(WINDOW *screen, struct flow_list *fl)
 {
-	int row_width = show_src ? 2 : 1;
-	unsigned int flows = 0;
-	unsigned int line = 4;
-	int skip = skip_lines;
-	struct flow_entry *n;
-
 	rcu_read_lock();
 
 	if (cds_list_empty(&fl->head))
-		mvwprintw(screen, line, 2, "(No sessions! "
+		mvwprintw(screen, 4, 2, "(No sessions! "
 			  "Is netfilter running?)");
 
-	ui_table_clear(&flows_tbl);
-	ui_table_header_print(&flows_tbl);
-
-	cds_list_for_each_entry_rcu(n, &fl->head, entry) {
-		if (!n->is_visible)
-			continue;
-		if (presenter_flow_wrong_state(n))
-			continue;
-
-		/* count only flows which might be showed */
-		flows++;
-
-		if (line + row_width >= rows)
-			continue;
-		if (--skip >= 0)
-			continue;
-
-		draw_flow_entry(n);
-		line += row_width;
-	}
+	ui_table_data_bind(&flows_tbl);
 
 	rcu_read_unlock();
 
-	mvwprintw(screen, 1, 0, "%*s", COLS - 1, " ");
-	mvwprintw(screen, 1, 2, "Kernel netfilter flows(%u) for ", flows);
-
-	draw_filter_status("Kernel netfilter flows", flows, skip_lines);
+	draw_filter_status(&flows_tbl, "Kernel netfilter flows");
 }
 
-static void draw_proc_entry(struct proc_entry *p)
+static void draw_proc_entry(struct ui_table *tbl, const void *data)
 {
-	struct ui_table *tbl = &procs_tbl;;
+	const struct proc_entry *p = data;
 	char tmp[128];
 
 	ui_table_row_add(tbl);
@@ -1209,51 +1192,33 @@ static void draw_proc_entry(struct proc_entry *p)
 	ui_table_row_col_set(tbl, TBL_PROC_FLOWS, tmp);
 
 	/* Bytes Src */
-	bandw2str(p->bytes_src, tmp, sizeof(tmp) - 1);
+	bandw2str(p->stat.bytes_src, tmp, sizeof(tmp) - 1);
 	ui_table_row_col_set(tbl, TBL_PROC_BYTES_SRC, tmp);
 
 	/* Rate Src */
-	rate2str(p->rate_bytes_src, tmp, sizeof(tmp) - 1);
+	rate2str(p->stat.rate_bytes_src, tmp, sizeof(tmp) - 1);
 	ui_table_row_col_set(tbl, TBL_PROC_RATE_SRC, tmp);
 
 	/* Bytes Dest */
-	bandw2str(p->bytes_dst, tmp, sizeof(tmp) - 1);
+	bandw2str(p->stat.bytes_dst, tmp, sizeof(tmp) - 1);
 	ui_table_row_col_set(tbl, TBL_PROC_BYTES_DST, tmp);
 
 	/* Rate Dest */
-	rate2str(p->rate_bytes_dst, tmp, sizeof(tmp) - 1);
+	rate2str(p->stat.rate_bytes_dst, tmp, sizeof(tmp) - 1);
 	ui_table_row_col_set(tbl, TBL_PROC_RATE_DST, tmp);
 
 	ui_table_row_show(tbl);
 }
 
-static void draw_procs(WINDOW *screen, struct flow_list *fl,
-		       int skip_lines)
+static void draw_procs(WINDOW *screen, struct flow_list *fl)
 {
-	struct proc_entry *proc, *tmp;
-	unsigned int line = 4;
-	int skip = skip_lines;
-	int procs = 0;
-
 	rcu_read_lock();
 
-	ui_table_clear(&procs_tbl);
-	ui_table_header_print(&procs_tbl);
-
-	cds_list_for_each_entry_safe(proc, tmp, &proc_list.head, entry) {
-		if (line + 1 >= rows)
-			continue;
-		if (--skip >= 0)
-			continue;
-
-		draw_proc_entry(proc);
-		procs++;
-		line++;
-	}
+	ui_table_data_bind(&procs_tbl);
 
 	rcu_read_unlock();
 
-	draw_filter_status("Processes", procs, skip_lines);
+	draw_filter_status(&procs_tbl, "Processes");
 }
 
 static void draw_help(void)
@@ -1355,6 +1320,17 @@ static void show_option_toggle(int opt)
 	}
 }
 
+void * flows_iter(void *data)
+{
+	struct flow_entry *n = data;
+
+	do {
+		n = list_first_or_next(n, &flow_list.head, entry);
+	} while (n && (!n->is_visible || presenter_flow_wrong_state(n)));
+
+	return n;
+}
+
 static void flows_table_init(struct ui_table *tbl)
 {
 	ui_table_init(tbl);
@@ -1382,6 +1358,16 @@ static void flows_table_init(struct ui_table *tbl)
 	ui_table_col_color_set(tbl, TBL_FLOW_STATE, COLOR(YELLOW, BLACK));
 
 	ui_table_header_color_set(&flows_tbl, COLOR(BLACK, GREEN));
+
+	ui_table_data_bind_set(tbl, draw_flow_entry);
+	ui_table_data_iter_set(tbl, flows_iter);
+}
+
+void * procs_iter(void *data)
+{
+	struct proc_entry *p = data;
+
+	return list_first_or_next(p, &proc_list.head, entry);
 }
 
 static void procs_table_init(struct ui_table *tbl)
@@ -1413,6 +1399,9 @@ static void procs_table_init(struct ui_table *tbl)
 	ui_table_col_color_set(tbl, TBL_PROC_RATE_DST, COLOR(BLUE, BLACK));
 
 	ui_table_header_color_set(tbl, COLOR(BLACK, GREEN));
+
+	ui_table_data_bind_set(tbl, draw_proc_entry);
+	ui_table_data_iter_set(tbl, procs_iter);
 }
 
 static void tab_main_on_open(struct ui_tab *tab, enum ui_tab_event_t evt, uint32_t id)
@@ -1420,10 +1409,13 @@ static void tab_main_on_open(struct ui_tab *tab, enum ui_tab_event_t evt, uint32
 	if (evt != UI_TAB_EVT_OPEN)
 		return;
 
-	if (id == TAB_FLOWS)
-		draw_flows(screen, &flow_list, skip_lines);
-	else if (id == TAB_PROCS)
-		draw_procs(screen, &flow_list, skip_lines);
+	if (id == TAB_FLOWS) {
+		draw_flows(screen, &flow_list);
+		curr_tbl = &flows_tbl;
+	} else if (id == TAB_PROCS) {
+		draw_procs(screen, &flow_list);
+		curr_tbl = &procs_tbl;
+	}
 }
 
 static void presenter(void)
@@ -1453,7 +1445,7 @@ static void presenter(void)
 	ui_tab_pos_set(tab_main, 2, 0);
 	ui_tab_active_color_set(tab_main, COLOR(BLACK, GREEN));
 	ui_tab_entry_add(tab_main, TAB_FLOWS, "Flows");
-	ui_tab_entry_add(tab_main, TAB_PROCS, "Process");
+	ui_tab_entry_add(tab_main, TAB_PROCS, "Processes");
 
 	rcu_register_thread();
 	while (!sigint) {
@@ -1470,24 +1462,20 @@ static void presenter(void)
 		case KEY_UP:
 		case 'u':
 		case 'k':
-			skip_lines--;
-			if (skip_lines < 0)
-				skip_lines = 0;
+			ui_table_event_send(curr_tbl, UI_EVT_SCROLL_UP);
 			break;
 		case KEY_DOWN:
 		case 'd':
 		case 'j':
-			skip_lines++;
-			if (skip_lines > SCROLL_MAX)
-				skip_lines = SCROLL_MAX;
+			ui_table_event_send(curr_tbl, UI_EVT_SCROLL_DOWN);
 			break;
 		case KEY_LEFT:
 		case 'h':
-			ui_table_event_send(&flows_tbl, UI_EVT_SCROLL_LEFT);
+			ui_table_event_send(curr_tbl, UI_EVT_SCROLL_LEFT);
 			break;
 		case KEY_RIGHT:
 		case 'l':
-			ui_table_event_send(&flows_tbl, UI_EVT_SCROLL_RIGHT);
+			ui_table_event_send(curr_tbl, UI_EVT_SCROLL_RIGHT);
 			break;
 		case 'b':
 			if (rate_type == RATE_BYTES)
@@ -1598,7 +1586,7 @@ static void conntrack_tstamp_enable(void)
 
 static void flow_entry_filter(struct flow_entry *n)
 {
-	if (show_active_only && !n->rate_bytes_src && !n->rate_bytes_dst)
+	if (show_active_only && !n->stat.rate_bytes_src && !n->stat.rate_bytes_dst)
 		n->is_visible = false;
 	else
 		n->is_visible = true;
@@ -1657,16 +1645,16 @@ static void collector_refresh_procs(void)
 			continue;
 		}
 
-		p->rate_bytes_src = 0;
-		p->rate_bytes_dst = 0;
-		p->rate_pkts_src = 0;
-		p->rate_pkts_dst = 0;
+		p->stat.rate_bytes_src = 0;
+		p->stat.rate_bytes_dst = 0;
+		p->stat.rate_pkts_src = 0;
+		p->stat.rate_pkts_dst = 0;
 
 		cds_list_for_each_entry_rcu(n, &p->flows, proc_head) {
-			p->rate_bytes_src += n->rate_bytes_src;
-			p->rate_bytes_dst += n->rate_bytes_dst;
-			p->rate_pkts_src += n->rate_pkts_src;
-			p->rate_pkts_dst += n->rate_pkts_dst;
+			p->stat.rate_bytes_src += n->stat.rate_bytes_src;
+			p->stat.rate_bytes_dst += n->stat.rate_bytes_dst;
+			p->stat.rate_pkts_src += n->stat.rate_pkts_src;
+			p->stat.rate_pkts_dst += n->stat.rate_pkts_dst;
 		}
 	}
 }
