@@ -1196,12 +1196,6 @@ static void init_ctx(struct ctx *ctx)
 	ctx->promiscuous = true;
 	ctx->randomize = false;
 	ctx->hwtimestamp = true;
-
-	ctx->pkts_recvd = 0;
-	ctx->pkts_seen = 0;
-	ctx->pkts_drops = 0;
-	ctx->pkts_recvd_last = 0;
-	ctx->pkts_drops_last = 0;
 }
 
 static void destroy_ctx(struct ctx *ctx)
@@ -1258,7 +1252,7 @@ static void __noreturn help(void)
 	     "  -v|--version                   Show version and exit\n"
 	     "  -h|--help                      Guess what?!\n\n"
 	     "Examples:\n"
-	     "  netsniff-ng --in eth0 --out dump.pcap -s -T 0xa1b2c3d4 --b 0 tcp or udp\n"
+	     "  netsniff-ng --in eth0 --out dump.pcap -s -T 0xa1b2c3d4 --bind-cpu 0 tcp or udp\n"
 	     "  netsniff-ng --in wlan0 --rfraw --out dump.pcap --silent --bind-cpu 0\n"
 	     "  netsniff-ng --in dump.pcap --mmap --out eth0 -k1000 --silent --bind-cpu 0\n"
 	     "  netsniff-ng --in dump.pcap --out dump.cfg --silent --bind-cpu 0\n"
