@@ -45,7 +45,6 @@
 #include "lockme.h"
 #include "privs.h"
 #include "proc.h"
-#include "mac80211.h"
 #include "ioops.h"
 #include "irq.h"
 #include "config.h"
@@ -57,6 +56,7 @@
 #include "csum.h"
 #include "trafgen_proto.h"
 #include "pcap_io.h"
+#include "trafgen_dev.h"
 
 enum shaper_type {
 	SHAPER_NONE,
@@ -79,10 +79,12 @@ struct shaper {
 struct ctx {
 	bool rand, rfraw, jumbo_support, verbose, smoke_test, enforce, qdisc_path;
 	size_t reserve_size;
+	struct dev_io *dev_out;
+	struct dev_io *dev_in;
 	unsigned long num;
 	unsigned int cpus;
 	uid_t uid; gid_t gid;
-	char *device, *device_trans, *rhost;
+	char *device, *rhost;
 	struct sockaddr_in dest;
 	struct shaper sh;
 	char *packet_str;
@@ -145,7 +147,6 @@ static const char *copyright = "Please report bugs to <netsniff-ng@googlegroups.
 	"This is free software: you are free to change and redistribute it.\n"
 	"There is NO WARRANTY, to the extent permitted by law.";
 
-static int sock;
 static struct cpu_stats *stats;
 static unsigned int seed;
 
@@ -184,31 +185,31 @@ static void __noreturn help(void)
 	puts("http://www.netsniff-ng.org\n\n"
 	     "Usage: trafgen [options] [packet]\n"
 	     "Options:\n"
-	     "  -i|-c|--in|--conf <cfg/->      Packet configuration file/stdin\n"
-	     "  -o|-d|--out|--dev <netdev>     Networking device i.e., eth0\n"
-	     "  -p|--cpp                       Run packet config through C preprocessor\n"
-	     "  -D|--define                    Add macro/define for C preprocessor\n"
-	     "  -J|--jumbo-support             Support 64KB super jumbo frames (def: 2048B)\n"
-	     "  -R|--rfraw                     Inject raw 802.11 frames\n"
-	     "  -s|--smoke-test <ipv4>         Probe if machine survived fuzz-tested packet\n"
-	     "  -n|--num <uint>                Number of packets until exit (def: 0)\n"
-	     "  -r|--rand                      Randomize packet selection (def: round robin)\n"
-	     "  -P|--cpus <uint>               Specify number of forks(<= CPUs) (def: #CPUs)\n"
-	     "  -t|--gap <time>                Set approx. interpacket gap (s/ms/us/ns, def: us)\n"
-	     "  -b|--rate <rate>               Send traffic at specified rate (pps/B/kB/MB/GB/kbit/Mbit/Gbit/KiB/MiB/GiB)\n"
-	     "  -S|--ring-size <size>          Manually set mmap size (KiB/MiB/GiB)\n"
-	     "  -E|--seed <uint>               Manually set srand(3) seed\n"
-	     "  -u|--user <userid>             Drop privileges and change to userid\n"
-	     "  -g|--group <groupid>           Drop privileges and change to groupid\n"
-	     "  -H|--prio-high                 Make this high priority process\n"
-	     "  -A|--no-sock-mem               Don't tune core socket memory\n"
-	     "  -Q|--notouch-irq               Do not touch IRQ CPU affinity of NIC\n"
-	     "  -q|--qdisc-path                Enable qdisc kernel path (default off since 3.14)\n"
-	     "  -V|--verbose                   Be more verbose\n"
-	     "  -C|--no-cpu-stats              Do not print CPU time statistics on exit\n"
-	     "  -v|--version                   Show version and exit\n"
-	     "  -e|--example                   Show built-in packet config example\n"
-	     "  -h|--help                      Guess what?!\n\n"
+	     "  -i|-c|--in|--conf <cfg/->             Packet configuration file/stdin\n"
+	     "  -o|-d|--out|--dev <netdev|.cfg|.pcap> Networking device or configuration file i.e., eth0\n"
+	     "  -p|--cpp                              Run packet config through C preprocessor\n"
+	     "  -D|--define                           Add macro/define for C preprocessor\n"
+	     "  -J|--jumbo-support                    Support 64KB super jumbo frames (def: 2048B)\n"
+	     "  -R|--rfraw                            Inject raw 802.11 frames\n"
+	     "  -s|--smoke-test <ipv4>                Probe if machine survived fuzz-tested packet\n"
+	     "  -n|--num <uint>                       Number of packets until exit (def: 0)\n"
+	     "  -r|--rand                             Randomize packet selection (def: round robin)\n"
+	     "  -P|--cpus <uint>                      Specify number of forks(<= CPUs) (def: #CPUs)\n"
+	     "  -t|--gap <time>                       Set approx. interpacket gap (s/ms/us/ns, def: us)\n"
+	     "  -b|--rate <rate>                      Send traffic at specified rate (pps/B/kB/MB/GB/kbit/Mbit/Gbit/KiB/MiB/GiB)\n"
+	     "  -S|--ring-size <size>                 Manually set mmap size (KiB/MiB/GiB)\n"
+	     "  -E|--seed <uint>                      Manually set srand(3) seed\n"
+	     "  -u|--user <userid>                    Drop privileges and change to userid\n"
+	     "  -g|--group <groupid>                  Drop privileges and change to groupid\n"
+	     "  -H|--prio-high                        Make this high priority process\n"
+	     "  -A|--no-sock-mem                      Don't tune core socket memory\n"
+	     "  -Q|--notouch-irq                      Do not touch IRQ CPU affinity of NIC\n"
+	     "  -q|--qdisc-path                       Enable qdisc kernel path (default off since 3.14)\n"
+	     "  -V|--verbose                          Be more verbose\n"
+	     "  -C|--no-cpu-stats                     Do not print CPU time statistics on exit\n"
+	     "  -v|--version                          Show version and exit\n"
+	     "  -e|--example                          Show built-in packet config example\n"
+	     "  -h|--help                             Guess what?!\n\n"
 	     "Examples:\n"
 	     "  trafgen --dev eth0 --conf trafgen.cfg\n"
 	     "  trafgen -e | trafgen -i - -o eth0 --cpp -n 1\n"
@@ -315,20 +316,10 @@ static void apply_counter(int id)
 		struct counter *counter = &packet_dyn[id].cnt[j];
 
 		val = counter->val - counter->min;
-
-		switch (counter->type) {
-		case TYPE_INC:
-			val = (val + counter->inc) % (counter->max - counter->min + 1);
-			break;
-		case TYPE_DEC:
-			val = (val - counter->inc) % (counter->min - counter->max + 1);
-			break;
-		default:
-			bug();
-		}
+		val = (val + counter->inc) % (counter->max - counter->min + 1);
 
 		counter->val = val + counter->min;
-		packets[id].payload[counter->off] = val;
+		packets[id].payload[counter->off] = counter->val;
 	}
 }
 
@@ -664,11 +655,6 @@ static void xmit_slowpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 	unsigned long num = 1, i = 0;
 	struct timeval start, end, diff;
 	unsigned long long tx_bytes = 0, tx_packets = 0;
-	struct sockaddr_ll saddr = {
-		.sll_family = PF_PACKET,
-		.sll_halen = ETH_ALEN,
-		.sll_ifindex = device_ifindex(ctx->device),
-	};
 
 	if (ctx->num > 0)
 		num = ctx->num;
@@ -688,8 +674,7 @@ static void xmit_slowpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 	while (likely(sigint == 0 && num > 0 && plen > 0)) {
 		packet_apply_dyn_elements(i);
 retry:
-		ret = sendto(sock, packets[i].payload, packets[i].len, 0,
-			     (struct sockaddr *) &saddr, sizeof(saddr));
+		ret = dev_io_write(ctx->dev_out, &packets[i]);
 		if (unlikely(ret < 0)) {
 			if (errno == ENOBUFS) {
 				sched_yield();
@@ -745,15 +730,16 @@ retry:
 
 static void xmit_fastpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned long orig_num)
 {
-	int ifindex = device_ifindex(ctx->device);
+	int ifindex = dev_io_ifindex_get(ctx->dev_out);
 	uint8_t *out = NULL;
-	unsigned int it = 0;
+	unsigned int it = 0, retry = 100;
 	unsigned long num = 1, i = 0;
-	size_t size = ring_size(ctx->device, ctx->reserve_size);
+	size_t size = ring_size(dev_io_name_get(ctx->dev_out), ctx->reserve_size);
 	struct ring tx_ring;
 	struct frame_map *hdr;
 	struct timeval start, end, diff;
 	unsigned long long tx_bytes = 0, tx_packets = 0;
+	int sock = dev_io_fd_get(ctx->dev_out);
 
 	set_sock_prio(sock, 512);
 
@@ -815,7 +801,8 @@ static void xmit_fastpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 	bug_on(gettimeofday(&end, NULL));
 	timersub(&end, &start, &diff);
 
-	pull_and_flush_tx_ring_wait(sock);
+	while (pull_and_flush_tx_ring_wait(sock) < 0 && errno == ENOBUFS && retry-- > 0)
+		usleep(10000);
 	destroy_tx_ring(sock, &tx_ring);
 
 	stats[cpu].tx_packets = tx_packets;
@@ -938,69 +925,20 @@ static void xmit_packet_precheck(struct ctx *ctx, unsigned int cpu)
 	}
 }
 
-static void pcap_load_packets(const char *path)
+static void pcap_load_packets(struct dev_io *dev)
 {
-	const struct pcap_file_ops *pcap_io = pcap_ops[PCAP_OPS_SG];
-	uint32_t link_type, magic;
-	pcap_pkthdr_t phdr;
-	size_t buf_len;
-	uint8_t *buf;
-	int ret;
-	int fd;
+	struct packet *pkt;
 
-	fd = open(path, O_RDONLY | O_LARGEFILE | O_NOATIME);
-	if (fd < 0 && errno == EPERM)
-		fd = open_or_die(path, O_RDONLY | O_LARGEFILE);
-	if (fd < 0)
-		panic("Cannot open file %s! %s.\n", path, strerror(errno));
-
-	if (pcap_io->init_once_pcap)
-		pcap_io->init_once_pcap(false);
-
-	ret = pcap_io->pull_fhdr_pcap(fd, &magic, &link_type);
-	if (ret)
-		panic("Error reading pcap header!\n");
-
-	if (pcap_io->prepare_access_pcap) {
-		ret = pcap_io->prepare_access_pcap(fd, PCAP_MODE_RD, false);
-		if (ret)
-			panic("Error prepare reading pcap!\n");
-	}
-
-	buf_len = round_up(1024 * 1024, RUNTIME_PAGE_SIZE);
-	buf = xmalloc_aligned(buf_len, CO_CACHE_LINE_SIZE);
-
-	while (pcap_io->read_pcap(fd, &phdr, magic, buf, buf_len) > 0) {
-		struct packet *pkt;
-		size_t pkt_len;
-
-		pkt_len = pcap_get_length(&phdr, magic);
-		if (!pkt_len)
-			continue;
-
-		realloc_packet();
-
-		pkt = current_packet();
-
-		pkt->len = pkt_len;
-		pkt->payload = xzmalloc(pkt_len);
-		memcpy(pkt->payload, buf, pkt_len);
-		pcap_get_tstamp(&phdr, magic, &pkt->tstamp);
-	}
-
-	if (pcap_io->prepare_close_pcap)
-		pcap_io->prepare_close_pcap(fd, PCAP_MODE_RD);
-
-	free(buf);
-	close(fd);
+	while ((pkt = dev_io_read(dev)) != 0)
+		/* nothing to do */;
 }
 
 static void main_loop(struct ctx *ctx, char *confname, bool slow,
 		      unsigned int cpu, bool invoke_cpp, char **cpp_argv,
 		      unsigned long orig_num)
 {
-	if (ctx->pcap_in) {
-		pcap_load_packets(ctx->pcap_in);
+	if (ctx->dev_in && dev_io_is_pcap(ctx->dev_in)) {
+		pcap_load_packets(ctx->dev_in);
 		shaper_set_tstamp(&ctx->sh, &packets[0].tstamp);
 		ctx->num = plen;
 	} else {
@@ -1029,18 +967,15 @@ static void main_loop(struct ctx *ctx, char *confname, bool slow,
 		fflush(stdout);
 	}
 
-	sock = pf_socket();
-
-	if (ctx->qdisc_path == false)
-		set_sock_qdisc_bypass(sock, ctx->verbose);
+	dev_io_open(ctx->dev_out);
+	if (dev_io_is_netdev(ctx->dev_out) && ctx->qdisc_path == false)
+		set_sock_qdisc_bypass(dev_io_fd_get(ctx->dev_out), ctx->verbose);
 
 	if (slow)
 		xmit_slowpath_or_die(ctx, cpu, orig_num);
 	else
 		xmit_fastpath_or_die(ctx, cpu, orig_num);
 
-	close(sock);
-
 	cleanup_packets();
 }
 
@@ -1061,7 +996,7 @@ static unsigned int generate_srand_seed(void)
 
 static void on_panic_del_rfmon(void *arg)
 {
-	leave_rfmon_mac80211(arg);
+	dev_io_close(arg);
 }
 
 int main(int argc, char **argv)
@@ -1123,7 +1058,7 @@ int main(int argc, char **argv)
 			break;
 		case 'd':
 		case 'o':
-			ctx.device = xstrndup(optarg, IFNAMSIZ);
+			ctx.device = xstrdup(optarg);
 			break;
 		case 'H':
 			prio_high = true;
@@ -1152,15 +1087,13 @@ int main(int argc, char **argv)
 		case 'J':
 			ctx.jumbo_support = true;
 			break;
+		case 'c':
 		case 'i':
 			confname = xstrdup(optarg);
-			if (strstr(confname, ".pcap")) {
+			if (c == 'i' && strstr(confname, ".pcap")) {
 				ctx.sh.type = SHAPER_TSTAMP;
 				ctx.pcap_in = confname;
-				break;
-			}
-		case 'c':
-			if (!strncmp("-", confname, strlen("-")))
+			} else if (!strncmp("-", confname, strlen("-")))
 				ctx.cpus = 1;
 			break;
 		case 'u':
@@ -1306,16 +1239,12 @@ int main(int argc, char **argv)
 		panic("No networking device given!\n");
 	if (confname == NULL && !ctx.packet_str)
 		panic("No configuration file or packet string given!\n");
-	if (device_mtu(ctx.device) == 0)
-		panic("This is no networking device!\n");
 
 	register_signal(SIGINT, signal_handler);
 	register_signal(SIGQUIT, signal_handler);
 	register_signal(SIGTERM, signal_handler);
 	register_signal(SIGHUP, signal_handler);
 
-	protos_init(ctx.device);
-
 	if (prio_high) {
 		set_proc_prio(-20);
 		set_sched_status(SCHED_FIFO, sched_get_priority_max(SCHED_FIFO));
@@ -1325,16 +1254,29 @@ int main(int argc, char **argv)
 		set_system_socket_memory(vals, array_size(vals));
 	xlockme();
 
-	if (ctx.rfraw) {
-		ctx.device_trans = xstrdup(ctx.device);
-		xfree(ctx.device);
-
-		enter_rfmon_mac80211(ctx.device_trans, &ctx.device);
-		panic_handler_add(on_panic_del_rfmon, ctx.device);
-		sleep(0);
+	if (ctx.pcap_in) {
+		ctx.dev_in = dev_io_create(ctx.pcap_in, DEV_IO_IN);
+		if (!ctx.dev_in)
+			panic("Failed to open input device\n");
+		dev_io_open(ctx.dev_in);
 	}
 
-	if (shaper_is_set(&ctx.sh) || (ctx.pcap_in)) {
+	ctx.dev_out = dev_io_create(ctx.device, DEV_IO_OUT);
+	if (!ctx.dev_out)
+		panic("Failed to open output device\n");
+
+	if (ctx.rfraw) {
+		if (dev_io_link_type_set(ctx.dev_out, LINKTYPE_IEEE802_11_RADIOTAP))
+			panic("Failed to setup rfraw device\n");
+
+		panic_handler_add(on_panic_del_rfmon, ctx.dev_out);
+	}
+
+	protos_init(ctx.dev_out);
+
+	if (shaper_is_set(&ctx.sh) || (ctx.dev_in && !dev_io_is_netdev(ctx.dev_in))
+	    || !dev_io_is_netdev(ctx.dev_out)) {
+
 		prctl(PR_SET_TIMERSLACK, 1UL);
 		/* Fall back to single core to not mess up correct timing.
 		 * We are slow anyway!
@@ -1351,9 +1293,10 @@ int main(int argc, char **argv)
 	if (ctx.num)
 		ctx.cpus = min_t(unsigned int, ctx.num, ctx.cpus);
 
-	irq = device_irq_number(ctx.device);
-	if (set_irq_aff)
+	if (set_irq_aff && dev_io_is_netdev(ctx.dev_out)) {
+		irq = device_irq_number(ctx.device);
 		device_set_irq_affinity_list(irq, 0, ctx.cpus - 1);
+	}
 
 	stats = setup_shared_var(ctx.cpus);
 
@@ -1384,9 +1327,6 @@ int main(int argc, char **argv)
 			die();
 	}
 
-	if (ctx.rfraw)
-		leave_rfmon_mac80211(ctx.device);
-
 	if (set_sock_mem)
 		reset_system_socket_memory(vals, array_size(vals));
 
@@ -1411,12 +1351,15 @@ int main(int argc, char **argv)
 thread_out:
 	xunlockme();
 	destroy_shared_var(stats, ctx.cpus);
-	if (set_irq_aff)
+	if (dev_io_is_netdev(ctx.dev_out) && set_irq_aff)
 		device_restore_irq_affinity_list();
 
+	dev_io_close(ctx.dev_out);
+	if (ctx.dev_in)
+		dev_io_close(ctx.dev_in);
+
 	argv_free(cpp_argv);
 	free(ctx.device);
-	free(ctx.device_trans);
 	free(ctx.rhost);
 	free(confname);
 	free(ctx.packet_str);
